import { Plugin } from 'vite';
type ReplaceFn = (source: string, path: string) => string;
type ReplacePair = {
    from: RegExp | string | string[];
    to: string | number;
};
interface Replacement {
    /**
     * for debugging purpose
     */
    id?: string | number;
    /**
     * name matching of module
     */
    filter: RegExp | string | string[];
    /**
     * esm modules which required by using `require('esm')`
     */
    replace: ReplacePair | ReplaceFn | Array<ReplacePair | ReplaceFn>;
}
interface Options {
    /**
     * Enforce plugin invocation tier similar to webpack loaders.
     *
     * Plugin invocation order:
     * - alias resolution
     * - `enforce: 'pre'` plugins
     * - vite core plugins
     * - normal plugins
     * - vite build plugins
     * - `enforce: 'post'` plugins
     * - vite build post plugins
     */
    enforce?: 'pre' | 'post';
    /**
     * Apply the plugin only for serve or for build.
     */
    apply?: 'serve' | 'build';
}
export default function filterReplace(replacements?: Replacement[], options?: Options): Plugin;
export {};
